local L = AceLibrary("AceLocale-2.2"):new("Quartz")

local QuartzGCD = Quartz:NewModule('GCD')
local Quartz = Quartz

local GetTime = GetTime

local gcdbar, gcdbar_width, gcdspark, db
local starttime = 0
local duration = 1.5

local function OnUpdate()
	gcdspark:ClearAllPoints()
	local perc = (GetTime() - starttime) / duration
	if perc > 1 then
		return gcdbar:Hide()
	else
		gcdspark:SetPoint('CENTER', gcdbar, 'LEFT', gcdbar_width * perc, 0)
	end
end
local function OnHide()
	gcdbar:SetScript('OnUpdate', nil)
end
local function OnShow()
	gcdbar:SetScript('OnUpdate', OnUpdate)
end

function QuartzGCD:OnInitialize()
	db = Quartz:AcquireDBNamespace("GCD")
	Quartz:RegisterDefaults("GCD", "profile", {
		sparkcolor = {1, 1, 1},
		gcdalpha = 0.9,
		gcdheight = 4,
	})
end
function QuartzGCD:OnEnable()
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	if not gcdbar then
		gcdbar = CreateFrame('Frame', 'QuartzGCDBar', UIParent)
		gcdbar:SetFrameStrata('HIGH')
		gcdbar:SetScript('OnShow', OnShow)
		gcdbar:SetScript('OnHide', OnHide)
		
		gcdspark = gcdbar:CreateTexture(nil, 'DIALOG')
		gcdbar:Hide()
	end
	Quartz.ApplySettings()
end
function QuartzGCD:OnDisable()
	gcdbar:Hide()
end

function QuartzGCD:UNIT_SPELLCAST_SENT(unit)
	if unit == 'player' then
		local time = GetTime()
		if time - starttime >= 1.3 then
			starttime = time
			gcdbar:Show()
		end
	end
end
function QuartzGCD:ApplySettings()
	if gcdbar and Quartz:IsModuleActive('GCD') then
		gcdbar:ClearAllPoints()
		gcdbar:SetPoint("TOP", QuartzCastBar, "BOTTOM", 0, 2)
		gcdbar:SetHeight(db.profile.gcdheight)
		gcdbar_width = QuartzCastBar:GetWidth() - 8
		gcdbar:SetWidth(gcdbar_width)
		gcdbar:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16})
		gcdbar:SetBackdropColor(0,0,0)
		gcdbar:SetAlpha(db.profile.gcdalpha)
		
		gcdspark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
		gcdspark:SetVertexColor(unpack(db.profile.sparkcolor))
		gcdspark:SetBlendMode('ADD')
		gcdspark:SetWidth(15)
		gcdspark:SetHeight(db.profile.gcdheight*2.5)
	end
end
do
	local function set(field, value)
		db.profile[field] = value
		Quartz.ApplySettings()
	end
	local function get(field)
		return db.profile[field]
	end
	local function setcolor(field, ...)
		db.profile[field] = {...}
		Quartz.ApplySettings()
	end
	local function getcolor(field)
		return unpack(db.profile[field])
	end
	Quartz.options.args.modules.args.gcd = {
		type = 'group',
		name = L["Global Cooldown"],
		desc = L["Global Cooldown"],
		args = {
			toggle = {
				type = 'toggle',
				name = L["Enable"],
				desc = L["Enable"],
				get = function()
					return Quartz:IsModuleActive('GCD')
				end,
				set = function(v)
					Quartz:ToggleModuleActive('GCD', v)
				end,
				order = 100,
			},
			gcdcolor = {
				type = 'color',
				name = L["Spark Color"],
				desc = L["Spark Color"],
				get = getcolor,
				set = setcolor,
				passValue = 'sparkcolor',
				order = 101,
			},
			gcdheight = {
				type = 'range',
				name = L["Height"],
				desc = L["Height"],
				min = 2,
				max = 10,
				step = 1,
				get = get,
				set = set,
				passValue = 'gcdheight',
				order = 102,
			},
			gcdalpha = {
				type = 'range',
				name = L["Alpha"],
				desc = L["Alpha"],
				min = 0.05,
				max = 1,
				step = 0.05,
				isPercent = true,
				get = get,
				set = set,
				passValue = 'gcdalpha',
				order = 103,
			},
		},
	}
end